/*Written by Vadim Teverovsky*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#define BIG 1.0e30
#define AFAC 1.5
#define AMP 1.5
double xmed;

void Cpeng(data, n, nvar, minpts, v)
double *data, *v;
long *n, *nvar, *minpts;

{
  int N = (int)*n, NVAR = (int)*nvar;

  int i, j, m, npoints;
  double increment, grandmean;
  double mean(), variance(), lsfit();
  void median();

/* dynamically allocate storage to var*/

  double *var, *residual;
  var = (double *) calloc(N+1,sizeof(double));
  residual = (double *) calloc(N+1, sizeof(double));

  increment = (log10((double) N/3))/NVAR;
  v[0]=0;
  for(i = 1; i < NVAR + 1; i++)
    {
    v[i] = 0;
    m = 3*(int)floor(pow(10.,(i * increment)));
/*    npoints = (int)floor((double) N - m - 1);*/
    npoints = (int)floor((double) N/m);
    for ( j = 1; j <= npoints; j++){
      var[j] = lsfit(data, j-1, m, residual);
/*      v[i] += var[j]/npoints; */
    }
    median(var,npoints);
/*    if (npoints < 100) {*/
/*      for ( j = 1; j <= npoints; j++){*/
/*	printf("var[%d] = %f \n", j, var[j]);}*/
/*}*/
/*    printf("m = %d, npoints = %d, v[%d] = %f, med = %f\n", m,npoints,i,v[i],xmed);*/
    v[i] = xmed;


  }
/* deallocate space pointed to by var */

  free(var);
  free(residual);
}

void
median(x, n)
int n;
double *x;
{
  int i, np, nm, j;
  double xx, xp, xm, sumx, sum , eps, stemp, dum, ap, am, aa, a;
  
  a = 0.5*(x[1]+x[n]);
  eps = fabs(x[n]-x[1]);
  am = -(ap=BIG/2);
  for(i=1;;i++){
    sum=sumx=0.0;
    np=nm=0;
    xm = -(xp=BIG/2);

    for(j = 1; j <= n; j++){
      xx = x[j];
      if ( xx != a) {
	if (xx>a){
	  ++np;
	  if(xx<xp) xp = xx;
	} else if ( xx < a) {
	  ++nm;
	  if ( xx > xm) xm = xx;
	}
	sum += dum=1.0/(eps+fabs(xx-a));
	sumx += xx*dum;
      }
    }
    stemp=(sumx/sum)-a;
/*      printf("n = %d, ap = %f, am = %f  \n", n, ap, am);*/
/*	printf("aa=%f, a = %f, np = %d, nm = %d\n", a, aa, np, nm);*/
    if (np-nm >=2) {
      am = a;
      aa = stemp < 0.0 ? xp : xp+stemp*AMP;
      if (aa>ap) aa = 0.5*(a+ap);
      eps = AFAC*fabs(aa-a);
      a = aa;
    } else if (nm - np >=2) {
      ap =a;
      aa = stemp > 0.0 ? xm : xm+stemp*AMP;
      if (aa<am) aa = 0.5*(a+am);
      eps = AFAC*fabs(aa-a);
      a = aa;
    } else {
      if ( n % 2 == 0) {
	xmed = 0.5*(np == nm ? xp+xm : np > nm ? a+xp : xm + a);
      } else {
 	xmed = np == nm ? a : np > nm ? xp : xm;
      }
      return;
    }
    if ( (((ap-am)/am <= .01) && ((ap-am)/ap <=.01)) || (i >=90)) {
      xmed = (ap+am)/2;
     printf("n = %d, ap = %f, am = %f, i = %d\n", n, ap, am, i);
      return;
    }
    
  }
}


double
lsfit(data, j, m, residual)
double *data, *residual;
int j, m;
{
  double variance(), sumy, sumx, sumx2, sumxy,a,b, vari;
  int  k;

  sumy = 0.;
  sumx = 0.;
  sumxy = 0.;
  sumx2 = 0.;

  residual[0] = data[j*m];
/*  residual[0] = data[j];*/
  for ( k = 1; k < m; k++){
    residual[k] = residual[k-1] + data[k+j*m];
  }
  for ( k = 0; k < m; k++){  
    sumy += residual[k];
    sumx += k;
    sumx2 += ((double) k) * ((double) k);
    sumxy += k*residual[k];
  }

  if ((sumx2-sumx*sumx) == 0) return(0);

  a = (sumxy- sumx*sumy/m)/(sumx2-sumx*sumx/m);
  b = sumy/m-sumx/m*a;
 /* printf("j = %d, m = %d, a= %f, b= %f \n", j, m, a, b);*/
  for ( k = 0; k < m; k++){
    residual[k] -= a*k+b;
  }
  vari = variance(residual, m);
  return(vari); 
}

double
mean(data, j, m)
double *data;
int j, m;
{
  double sum, average;
  int i, first, last;

  sum = 0.;
  first = m * j;
  last = m * (j + 1);
  for(i = first; i < last; i++)
    sum = sum + data[i];
  average = sum / m;
  return(average);
}

double
variance(averages, npoints)
double *averages;
int npoints;
{
  int i;
  double sumsq, var, ave, mean();

  ave = mean(averages, 0,npoints);
  sumsq = 0.;
  for(i = 0; i < npoints; i++)
    sumsq = sumsq + pow(averages[i]-ave, 2.);
  var = sumsq/(npoints-1);
  return(var);
}
