#include <math.h>
#include <stdio.h>
#include <stdlib.h>
/*Written by Bob Sherman, modified by Walter Willinger, Vadim Teverovsky.*/


#define  r(i,j)       output[(i) * NBLK + (j)]
#define  radj(i,j)    output[NBLK * NLAG + (i) * NBLK + (j)]

/* *** Crs
 * S feeds this function a time series (data) of length n.
 * The appropriate r and r/s statistics are computed and then
 * returned to S through the vector output.
 */

void
Crs (data, n, nblk, nlag, overlap, output)
     double *data;
     long *n, *nblk, *nlag, *overlap;
     double *output;

{
  int N = (int) *n, NBLK = (int) *nblk, NLAG = (int) *nlag;
  int OVERLAP = (int) *overlap, BLKSIZE;

  int i, j, k, d, correction, NVAL;

  double increment;
  double temp, min, max, s;
  double ave, secondmom;

/* Dynamically allocate storage to xcum and xsqcum. */

  double *xcum, *xsqcum;

  xcum = (double *) calloc (N, sizeof (double));
  xsqcum = (double *) calloc (N, sizeof (double));

#if defined (DEBUG)
  printf("you made it!\n\n");
#endif

  /* Compute xcum's and xsqcum's. */

  xcum[0] = data[0];
  xsqcum[0] = data[0] * data[0];

  for (i = 1; i < N; i++)
    {
      xcum[i] = xcum[i - 1] + data[i];
      xsqcum[i] = xsqcum[i - 1] + data[i] * data[i];
    }

  /* Compute r and radj. */

  BLKSIZE = (int) floor (((double) N) / ((double) NBLK));

  if (OVERLAP != 0)
    increment = (log10 ((double) N)) / NLAG;
  else
    increment = (log10 ((double) BLKSIZE)) / NLAG;

  for (k = 0; k < NLAG; k++)
    {
      if (k == NLAG - 1)
	d = (int) (pow (10., (double) (increment * (k + 1))));
      else
	d = (int) ceil (pow (10., (double) (increment * (k + 1))));

      /* d observations used to compute r and radj for lag k. */

      correction = (int) ceil (((double) (d - BLKSIZE)) / ((double) BLKSIZE));
      if (correction == NBLK)
	correction = correction - 1;
      else;

      if (d > BLKSIZE)
	NVAL = NBLK - correction;
      else
	NVAL = NBLK;

#if defined (DEBUG)
      printf("NVAL[%d] = %d  d[%d] = %d\n",k + 1,NVAL,k + 1,d);
#endif

      /* NVAL is the number of r and radj values computed for lag k. */

      /* i = 0 is a special case. */

      max = min = 0.;
      ave = (1. / d) * xcum[d - 1];
      for (j = 0; j < d; j++)
	{
	  temp = xcum[j] - (j + 1) * ave;
	  if (temp > max)
	    max = temp;
	  else if (temp < min)
	    min = temp;
	}
      r (k, 0) = max - min;
      secondmom = (1. / d) * xsqcum[d - 1];
      if (secondmom > ave * ave)
	{
	  s = sqrt (secondmom - ave * ave);
	  radj (k, 0) = r (k, 0) / s;
	}
      else
	{
	  radj (k, 0) = r (k, 0);
#if defined (DEBUG)
	  printf("s[%d][1] = 0 radj(%d, 1) set equal to r(%d, 1)\n",
		 k+1, k+1, k+1);
#endif
	}

      /* i > 0 */

      for (i = 1; i < NVAL; i++)
	{
	  max = min = 0.;
	  ave = (1. / d) * (xcum[BLKSIZE * i - 1 + d] - xcum[BLKSIZE * i - 1]);
	  for (j = 0; j < d; j++)
	    {
	      temp = xcum[BLKSIZE * i + j] - xcum[BLKSIZE * i - 1]
		- (j + 1) * ave;
	      if (temp > max)
		max = temp;
	      else if (temp < min)
		min = temp;
	    }
	  r (k, i) = max - min;
	  secondmom = (1. / d) * (xsqcum[BLKSIZE * i - 1 + d]
				  - xsqcum[BLKSIZE * i - 1]);
	  if (secondmom > ave * ave)
	    {
	      s = sqrt (secondmom - ave * ave);
	      radj (k, i) = r (k, i) / s;
	    }
	  else
	    {
	      radj (k, i) = r (k, i);
#if defined (DEBUG)
	      printf("s[%d][%d] = 0 radj(%d, %d) set equal to r(%d, %d)\n",
		     k+1, i+1, k+1, i+1, k+1, i+1);
#endif
	    }
	}
    }

  /* Deallocate space pointed to by xcum and xsqcum. */

  free (xcum);
  free (xsqcum);

}

/* eof Crs.c */

