/* Cvariances.c */
/* Written by Bob Sherman, modified by Walter Willinger, Vadim Teverovsky.*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>

/* S feeds this function a time series (data) of length n.  A vector of
   variances (v) is returned.  This vector has length nvar + 1 and its ith
   component is the variance of (the integer part of) n/m terms (m depends on 
   i and nvar and is defined below), each of which is the average of m terms.
 */

void
Cvariances (data, n, nvar, minpts, v)
     double *data;
     long *n, *nvar, *minpts;
     double *v;

{
  int N = (int) *n, NVAR = (int) *nvar;
  double increment, grandmean;
  int i, j, m, npoints;
  double *averages;

  double mean (), variance ();

  /* Dynamically allocate storage to averages. */

  averages = (double *) calloc (N, sizeof (double));

  increment = (log10 ((double) N / *minpts)) / NVAR;
  grandmean = mean (data, 0, N);
  v[0] = variance (data, grandmean, N);

  for (i = 1; i < NVAR + 1; i++)
    {
      m = (int) floor (pow (10., (i * increment)));
      npoints = (int) floor ((double) N / m);
      for (j = 0; j < npoints; j++)
	averages[j] = mean (data, j, m);
      v[i] = variance (averages, grandmean, npoints);
    }

  /* Deallocate space pointed to by averages. */

  free (averages);
}

double
mean (data, j, m)
     double *data;
     int j, m;
{
  double sum, average;
  int i, first, last;

  sum = 0.;
  first = m * j;
  last = m * (j + 1);
  for (i = first; i < last; i++)
    sum = sum + data[i];
  average = sum / m;
  return (average);
}

double
variance (averages, grandmean, npoints)
     double *averages;
     double grandmean;
     int npoints;
{
  int i;
  double sumsq, var;

  sumsq = 0.;
  for (i = 0; i < npoints; i++)
    sumsq = sumsq + pow (averages[i] - grandmean, 2.);
  var = sumsq / (npoints - 1);
  return (var);
}

/* eof Cvariances. */
