# Quickly adapted from the version in fSeries on the CRAN network
# By Richard G. Clegg richard@richardclegg.org (2005)

dyn.load("fdsim.so")

farimagen <- 
function(ar = c(0.5, -0.5), ma = 0.1, d= 0, n = 100,
innov = NULL, n.start = 100, start.innov = NULL, rand.gen = rnorm, 
rseed = NULL, ...) 

{   # A function implemented by Diethelm Wuertz

    # Description:
    #   Simulates an ARIMA Time Series Process
    
    # Example:
    #	armaSim(ar = c(0.5, -0.5), ma = 0.1, d= 0)
    #	armaSim(ar = c(0.5, -0.5), ma = 0.1, d= 0.2)
    #	armaSim(ar = 0,  ma = 0)
    #	armaSim(d = 0.2)
    
    # FUNCTION:
    
    
    	
    # Simulate:
    if (is.integer(rseed)) 
    	set.seed(rseed)
    if (is.null(innov)) 
    	innov = rand.gen(n, ...)
    n = length(innov) 
    if (is.null(start.innov)) 
    	start.innov = rand.gen(n, ...) 
    n.start = length(start.innov)

    # AR PART:
    p = length(ar)
    if (p == 1 && ar == 0) 
    	p = 0
    if (p) { 
	    minroots = min(Mod(polyroot(c(1, -ar))))
        if (minroots <= 1) stop("ar part of model is not stationary") 
    }
    
    # MA PART:
    q = length(ma)
    if (q == 1 && ma == 0) 
    	q = 0
    if (n.start < p + q) 
    	stop("burn-in must be as long as ar + ma")
    
    # DIFFERENCING:
   
        if (d != round(d) ) { 
	        TSMODEL = "FRACDIFF" 
	    } else { 
		    TSMODEL = "ARIMA" 
	    } 
	
    
    # ARMA:
    if (TSMODEL == "ARIMA") {
        x = ts(c(start.innov, innov), start = 1 - n.start) 
        if (length(ma)) x = filter(x, c(1,ma), sides = 1)
        if (length(ar)) x = filter(x, ar, method = "recursive")
        x = x[-(1:n.start)]
        if (d > 0) x = diffinv(x, differences = d) 
    }
        
    if (TSMODEL == "FRACDIFF") {
        if (p == 0) ar = 0
        if (q == 0) ma = 0
        mu = 0
        # Use Fortran Routine from R's contributed fracdiff package:
        # This is a BUILTIN function ...
        x = .Fortran("fdsim", as.integer(n), as.integer(p), as.integer(q), 
            as.double(ar), as.double(ma), as.double(d), 
            as.double(mu), as.double(rnorm(n + q)), x = double(n + q), 
            as.double(.Machine$double.xmin), as.double(.Machine$double.xmax), 
            as.double(.Machine$double.neg.eps), as.double(.Machine$double.eps)
            )$x[1:n] 
    }
               
    # Return Value:
    as.ts(x)
}

